#!/bin/bash
# ==================================================================
# NetTest - Network performance testing during long run
#           Perform send & receive bw mesurement
#           Perform ping latency mesurement
#           Perform dataloss lesurement during low & high load
# ==================================================================
# Pre-requisite
#   * Bw mesurement is based on iPerf tool : see www.iperf.fr
# ==================================================================
# Output
#   * CSV file with all the mesures per colomn and test sequence per 
#     line
#   * seq#;send bw;receive bw;%ping loss;ping latency;%ping loss
#
# ==================================================================
# (c) Disk_91 - see www.disk91.com
# Applicable License is GNU/GPL
# Feel free to use, distribute and modify
# ==================================================================

# ####################################
# Default script parameters
# ####################################

# --- Sleep time between two sequences in seconds
TIME_BET_SEQ=10

# --- iPerf mesure duration in seconds
IPERF_DUR=30

# --- Number of ping to send for test
PING_NUM=30

# --- Maximum wait time for ping in seconds
PING_WAIT=2

# --- iperf command
IPERF_CMD="/usr/local/bin/iperf"

# ####################################
# Sub Functions
# ####################################

# ------------------------------------
proceed_sequence() {
  # $1 is the target Ip destintation
  # $2 is the sequence number


  # -- execute ping loss in parallel of iperf test
  local res_loss=''
  local out_loss=`mktemp`
  ( sleep 2 ; ping -W ${PING_WAIT} -c ${IPERF_DUR} "$1" 2>&1 | grep loss ) > ${out_loss} & 
  local pid_loss=$!

  # -- execute iperf test
  local res_bw=''
  local i_Result=`${IPERF_CMD} -t ${IPERF_DUR} -f m -c "$1" 2>&1 | grep Mbits`
  if [ ! -z "${i_Result}" ] ; then
    res_bw=`echo ${i_Result} | sed -e 's/^.* \([0-9.]\+\) Mbits.*$/\1/'`
    res_bw=`echo ${res_bw} | sed -e 's/\./,/'`
  else
    res_bw='-1'
  fi

  # -- end of ping loss execution
  wait ${pid_loss}
  local l_Result=`cat ${out_loss}`
  rm ${out_loss}
  if [ ! -z "${l_Result}" ] ; then
     res_loss=`echo ${l_Result} | sed -e 's/^.* \([0-9]\+\)% packet loss.*$/\1/'`
     res_loss='0,0'${res_loss}
  else
     res_loss=-1
  fi

  sleep 1

  # -- execute ping (latency) test
  local res_lat=''
  local res_los1=''
  local p_Result=`ping -A -W ${PING_WAIT} -c ${PING_NUM} "$1" 2>&1 | grep -e "\(rtt\|loss\)"`
  if [ ! -z "${p_Result}" ] ; then
    res_lat=`echo ${p_Result} | grep rtt | cut -d '=' -f 2 | cut -d "/" -f 2`
    res_lat=`echo ${res_lat} | sed -e 's/\./,/'`
    res_los1=`echo ${p_Result} | grep loss | sed -e 's/^.* \([0-9]\+\)% packet loss.*$/\1/'`
    res_los1='0,0'${res_los1}
  else
    res_lat='-1'
    res_los1='-1'
  fi

  echo $2';'${res_bw}';'NA';'${res_loss}';'${res_lat}';'${res_los1}
}

# ------------------------------------
check_environment() {
   # --- search for iperf command
   if [ ! -x ${IPERF_CMD} ] ; then
      echo "ERROR - the iperf command has not been found in the system"
      echo "        you can indicate the path using --iperf-command option"
      exit 2
   fi 
   # --- check Target IP format
   if ! echo $1 | grep -e "^[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}$" >/dev/null ; then
      echo "ERROR - the IP adress provided ($1) is not matching a usual IP adress"
      exit 3
   fi
   # -- check for destination IP responding
   if ! ping -c 1 -w 1 $1 > /dev/null ; then
      echo "ERROR - the target host ($1) is not reachable (ping test failed)"
      exit 4
   fi

   # --- check for server side responding
   if  ( ${IPERF_CMD} -c $1 -t 1 2>&1 | grep failed ) &> /dev/null ; then
      echo "ERROR - the iperf test to target host ($1) did not succeed. Please make sure iperf has"
      echo "        been started on the target host (iperf -s command)"
      exit 5
   fi

   return 0
}


# ------------------------------------
usage() {
    echo "netTest tool - network performance mesurement during long run"
    echo "usage :"
    echo "   $1 [OPTIONS] TargetIP"
    echo "OPTIONS"
    echo " --help | -h : display this message then exit"
    echo " --iperf-command | -ipc : indicate is next argument the iperf full path/name to be used"
    echo "                         default is ${IPERF_CMD}"
    echo " "
    echo "TargetIP - Ip of the target system to perform test with. This system is running iperf as a server"
    echo "           command to start on it is : iperf -s"

    exit 1
}


# ####################################
# Main entry point
# ####################################
main() {
   if [ $# -lt 1 ] ; then
     usage $0
   fi

   # --- Proceed options
   while [ $# -gt 1 ] ; do
     case $1  in
       '-h'|'--help') usage $0;;

       '-ipc'|'--iperf-command')
                IPERF_CMD="$2"
		shift;;

       *) usage $0;;
     esac
     shift
   done
   TARGET_IP="$1" 
  
   if check_environment "${TARGET_IP}"; then
      local seq_num=0
      echo "seq;send_bw;rec_bw;fr_loss_hb;latency;fr_loss_lb"
      while true ; do     
         proceed_sequence "${TARGET_IP}" ${seq_num}
         sleep ${TIME_BET_SEQ}
      done
   fi

}
main $*
